@@ -1,5 +1,5 @@
 # -*- Mode: cperl; cperl-indent-level: 4 -*-
-# $Id: Harness.pm,v 1.14.2.5 2001/08/12 02:55:08 schwern Exp $
+# $Id: Harness.pm,v 1.14.2.8 2001/09/29 04:06:23 schwern Exp $
 
 package Test::Harness;
 
@@ -22,7 +22,7 @@ use vars qw($VERSION $Verbose $Switches $Have_Devel_Corestack $Curtest
 
 $Have_Devel_Corestack = 0;
 
-$VERSION = "2.00_00";
+$VERSION = 2.00_01;
 
 $ENV{HARNESS_ACTIVE} = 1;
 
@@ -128,18 +128,14 @@ will generate
     FAILED tests 1, 3, 6
     Failed 3/6 tests, 50.00% okay
 
+=item B<test names>
 
-=item B<$Test::Harness::verbose>
+Anything after the test number but before the # is considered to be
+the name of the test.
 
-The global variable $Test::Harness::verbose is exportable and can be
-used to let runtests() display the standard output of the script
-without altering the behavior otherwise.
+  ok 42 this is the name of the test
 
-=item B<$Test::Harness::switches>
-
-The global variable $Test::Harness::switches is exportable and can be
-used to set perl command line options used for running the test
-script(s). The default value is C<-w>.
+Currently, Test::Harness does nothing with this information.
 
 =item B<Skipping tests>
 
@@ -217,6 +213,38 @@ test script, please use a comment.
 =back
 
 
+=head2 Taint mode
+
+Test::Harness will honor the -T in the #! line on your test files.  So
+if you begin a test with:
+
+    #!perl -T
+
+the test will be run with taint mode on.
+
+
+=head2 Configuration variables.
+
+These variables can be used to configure the behavior of
+Test::Harness.  They are exported on request.
+
+=over 4
+
+=item B<$Test::Harness::verbose>
+
+The global variable $Test::Harness::verbose is exportable and can be
+used to let runtests() display the standard output of the script
+without altering the behavior otherwise.
+
+=item B<$Test::Harness::switches>
+
+The global variable $Test::Harness::switches is exportable and can be
+used to set perl command line options used for running the test
+script(s). The default value is C<-w>.
+
+=back
+
+
 =head2 Failure
 
 It will happen, your tests will fail.  After you mop up your ego, you
@@ -305,7 +333,7 @@ sub runtests {
     my($tot, $failedtests) = _run_all_tests(@tests);
     _show_results($tot, $failedtests);
 
-    my $ok = ($tot->{bad} == 0 && $tot->{max});
+    my $ok = _all_ok($tot);
 
     assert(($ok xor keys %$failedtests), 
            q{ok status jives with $failedtests});
@@ -315,6 +343,20 @@ sub runtests {
 
 =begin _private
 
+=item B<_all_ok>
+
+  my $ok = _all_ok(\%tot);
+
+Tells you if this test run is overall successful or not.
+
+=cut
+
+sub _all_ok {
+    my($tot) = shift;
+
+    return $tot->{bad} == 0 && ($tot->{max} || $tot->{skipped}) ? 1 : 0;
+}
+
 =item B<_globdir>
 
   my @files = _globdir $dir;
@@ -413,10 +455,11 @@ sub _run_all_tests {
     my @dir_files = _globdir $Files_In_Dir if defined $Files_In_Dir;
     my $t_start = new Benchmark;
 
+    my $width = _leader_width(@tests);
     foreach my $tfile (@tests) {
         $Strap->_reset_file_state;
 
-        my($leader, $ml) = _mk_leader($tfile);
+        my($leader, $ml) = _mk_leader($tfile, $width);
         print $leader;
 
         my $fh = _open_test($tfile);
@@ -553,22 +596,25 @@ sub _run_all_tests {
 
 =item B<_mk_leader>
 
-  my($leader, $ml) = _mk_leader($test_file);
+  my($leader, $ml) = _mk_leader($test_file, $width);
 
 Generates the 't/foo........' $leader for the given $test_file as well
 as a similar version which will overwrite the current line (by use of
 \r and such).  $ml may be empty if Test::Harness doesn't think you're
 on TTY.
 
+The $width is the width of the "yada/blah.." string.
+
 =cut
 
 sub _mk_leader {
-    my $te = shift;
-    chomp($te);      # XXX chomp?
+    my($te, $width) = @_;
+    chomp($te);
+    $te =~ s/\.\w+$/./;
 
     if ($^O eq 'VMS') { $te =~ s/^.*\.t\./\[.t./s; }
     my $blank = (' ' x 77);
-    my $leader = "$te" . '.' x (20 - length($te));
+    my $leader = "$te" . '.' x ($width - length($te));
     my $ml = "";
 
     $ml = "\r$blank\r$leader"
@@ -577,6 +623,29 @@ sub _mk_leader {
     return($leader, $ml);
 }
 
+=item B<_leader_width>
+
+  my($width) = _leader_width(@test_files);
+
+Calculates how wide the leader should be based on the length of the
+longest test name.
+
+=cut
+
+sub _leader_width {
+    my $maxlen = 0;
+    my $maxsuflen = 0;
+    foreach (@_) {
+        my $suf    = /\.(\w+)$/ ? $1 : '';
+        my $len    = length;
+        my $suflen = length $suf;
+        $maxlen    = $len    if $len    > $maxlen;
+        $maxsuflen = $suflen if $suflen > $maxsuflen;
+    }
+    # + 3 : we want three dots between the test name and the "ok"
+    return $maxlen + 3 - $maxsuflen;
+}
+
 
 sub _show_results {
     my($tot, $failedtests) = @_;
@@ -584,7 +653,7 @@ sub _show_results {
     my $pct;
     my $bonusmsg = _bonusmsg($tot);
 
-    if ($tot->{bad} == 0 && $tot->{max}) {
+    if (_all_ok($tot)) {
         print "All tests successful$bonusmsg.\n";
     } elsif (!$tot->{tests}){
         die "FAILED--no tests were run for some reason.\n";
@@ -594,9 +663,10 @@ sub _show_results {
             "alas--no output ever seen\n";
     } else {
         $pct = sprintf("%.2f", $tot->{good} / $tot->{tests} * 100);
+        my $percent_ok = 100*$tot->{ok}/$tot->{max};
         my $subpct = sprintf " %d/%d subtests failed, %.2f%% okay.",
                               $tot->{max} - $tot->{ok}, $tot->{max}, 
-                              100*$tot->{ok}/$tot->{max};
+                              $percent_ok;
 
         my($fmt_top, $fmt) = _create_fmts($failedtests);
 
@@ -679,8 +749,8 @@ sub _parse_test_line {
 
     my %result;
     if ( $Strap->_is_test($line, \%result) ) {
-        $test->{next} ||= 1;
-        my $this = $test->{next};
+        $test->{'next'} ||= 1;
+        my $this = $test->{'next'};
 
         my($not, $tnum) = (!$result{ok}, $result{number});
 
