@@ -1,11 +1,11 @@
 # -*- Mode: cperl; cperl-indent-level: 4 -*-
-# $Id: Straps.pm,v 1.1.2.4 2001/08/12 02:56:59 schwern Exp $
+# $Id: Straps.pm,v 1.1.2.6 2001/09/29 03:53:03 schwern Exp $
 
 package Test::Harness::Straps;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.01';
+$VERSION = '0.02';
 
 use Test::Harness::Assert;
 
@@ -33,9 +33,10 @@ Test::Harness::Straps - detailed analysis of test results
   my %results = $strap->analyze_fh($name, $test_filehandle);
   my %results = $strap->analyze_file($test_file);
 
+  # UNIMPLEMENTED
   my %total = $strap->total_results;
 
-  # Altering the behavior of the strap
+  # Altering the behavior of the strap  UNIMPLEMENTED
   my $verbose_output = $strap->dump_verbose();
   $strap->dump_verbose_fh($output_filehandle);
 
@@ -136,7 +137,7 @@ sub analyze {
         }
         elsif( $self->_is_test($line, \%result) ) {
             $totals{seen}++;
-            $result{number} = $self->{next} unless $result{number};
+            $result{number} = $self->{'next'} unless $result{number};
 
             my $pass = $result{ok};
 
@@ -169,7 +170,7 @@ sub analyze {
             $totals{summary}[$result{number}] = 
               {$self->_summarize($pass, \%result)};
 
-            $self->{next}++;
+            $self->{'next'}++;
 
             # XXX handle counter mismatch
 
@@ -207,8 +208,17 @@ use that name for the total report.
 sub analyze_file {
     my($self, $file) = @_;
 
-    # XXX fix up this error handling.
-    open(FILE, "$^X $file|") or die $!;
+    local $ENV{IFS} = '';
+
+    # If $^X is just 'perl' we'll have to figure it out from PATH.
+    # So rather than implicitly trusting PATH, we'll just save
+    # ourselves a lot of work and explicitly trust it.
+    local($ENV{PATH}) = $ENV{PATH} =~ /(.*)/;
+
+    unless( open(FILE, '-|') || exec $^X, $file ) {
+        # XXX fix up this error handling.
+        die $!;
+    }
 
     # XXX simple, stupid, easy for now.
     $self->analyze_fh($file, \*FILE);
@@ -308,23 +318,23 @@ my $Report_Re = <<'REGEX';
                   (not\ )?               # failure?
                   ok\b
                   (?:\s+(\d+))?         # optional test number
+                  \s*
                   (.*)                  # and the rest
 REGEX
 
 my $Extra_Re = <<'REGEX';
                  ^
-                 (?:\s+ -\ (.+?))?           # optional name
-                 (?:\s+ \#\s*(\w+)             # optional todo/skip
-                   (?:\s+ (.*?))?              #   and optional reason
-                 )?
+                  (.*?) (?:(?:[^\\]|^)# (.*))?
                  $
 REGEX
 
 sub _is_test {
     my($self, $line, $test) = @_;
 
+    # We pulverize the line down into pieces in three parts.
     if( my($not, $num, $extra)    = $line  =~ /$Report_Re/ox ) {
-        my($name, $type, $reason) = $extra =~ /$Extra_Re/ox if defined $extra;
+        my($name, $control) = split /(?:[^\\]|^)#/, $extra if $extra;
+        my($type, $reason)  = $control =~ /^\s*(\S+)(?:\s+(.*))?$/ if $control;
 
         $test->{number} = $num;
         $test->{ok}     = $not ? 0 : 1;
@@ -388,7 +398,7 @@ sub _reset_file_state {
     delete @{$self}{qw(max skip_all todo)};
     $self->{line}       = 0;
     $self->{saw_header} = 0;
-    $self->{next}       = 1;
+    $self->{'next'}       = 1;
 }
 
 =back
@@ -402,13 +412,15 @@ The %results returned from analyze() contain the following information:
 
   max               total tests which should have been run
   seen              total tests actually seen
-  skip_all          if the whole test was skipped, this will contain 
-                    the reason.
+  skip_all          if the whole test was skipped, this will 
+                      contain the reason.
 
-  ok                number of tests which passed (including todo and skips)
+  ok                number of tests which passed 
+                      (including todo and skips)
 
   todo              number of todo tests seen
-  bonus             number of todo tests which unexpectedly passed
+  bonus             number of todo tests which 
+                      unexpectedly passed
 
   skip              number of tests skipped
 
@@ -417,15 +429,16 @@ So a successful test should have max == seen == ok.
 
 There is one final item, the summary.
 
-  summary           an array ref reporting the result of each test
-                    looks like this:
+  summary           an array ref reporting the result of 
+                    each test looks like this:
 
-    $results{summary}[$test_num] = { ok         => is the test considered ok?
-                                     actual_ok  => did it literally say 'ok'?
-                                     name   => name of the test (if any)
-                                     type   => 'skip' or 'todo' (if any)
-                                     reason => reason for the above (if any)
-                                   };
+    $results{summary}[$test_num] = 
+            { ok        => is the test considered ok?
+              actual_ok => did it literally say 'ok'?
+              name      => name of the test (if any)
+              type      => 'skip' or 'todo' (if any)
+              reason    => reason for the above (if any)
+            };
 
   Element 0 is unused (but reserved for future use).
 
@@ -464,6 +477,9 @@ sub _summarize {
 
 =back
 
+=head1 NOTES
+
+Test::Harness::Straps will run fine under taint mode.
 
 =head1 AUTHOR
 
